
package org.act.manage.util;
import java.io.*;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;

public class XMLConfigUtil {
	
	public static Document loadDocumentFromFile(String fileName)
			throws Exception {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		DocumentBuilder db = dbf.newDocumentBuilder();
		FileInputStream fis = new FileInputStream(fileName);
		Document doc = db.parse(fis);
		fis.close();
		return doc;
	}
	
	public static Document newDocument() throws Exception {
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		DocumentBuilder db = dbf.newDocumentBuilder();
		return db.newDocument();
	}

	public static Vector getElement(Document doc, Element parent, String path,
			String namespaceURI) {
		String[] eles = path.split("/");
		if (parent == null)
			parent = doc.getDocumentElement();
		Element tmpEle = parent;
		Vector v = null;
		for (int i = 0; i < eles.length; i++) {
			v = XMLUtils.getChildElementsByNS(tmpEle, namespaceURI, eles[i]);
			if (v.size() < 1)
				return null;
			tmpEle = (Element) v.get(0);
		}
		return v;
	}
	
	public static Element newElement(Document doc, Element parent, String path,
			String namespaceURI) {
		String[] eles = path.split("/");
		if (parent == null)
			parent = doc.getDocumentElement();
		Element tmpEle = parent;
		Element newEle = null;
		Vector v = null;
				int i;
		for (i = 0; i < eles.length - 1; i++) {
			v = XMLUtils.getChildElementsByNS(tmpEle, namespaceURI, eles[i]);
			if (v.size() < 1) {
				newEle = doc.createElementNS(namespaceURI, eles[i]);
				tmpEle.appendChild(newEle);
				tmpEle = newEle;
			} else
				tmpEle = (Element) v.get(0);
		}
				newEle = doc.createElementNS(namespaceURI, eles[i]);
		tmpEle.appendChild(newEle);
		return newEle;
	}

	public static Element openElement(Document doc, Element parent,
			String path, String namespaceURI) {
		String[] eles = path.split("/");
		if (parent == null)
			parent = doc.getDocumentElement();
		Element tmpEle = parent;
		Element newEle = null;
		Vector v = null;
				int i;
		for (i = 0; i < eles.length; i++) {
			v = XMLUtils.getChildElementsByNS(tmpEle, namespaceURI, eles[i]);
			if (v.size() < 1) {
				newEle = doc.createElementNS(namespaceURI, eles[i]);
				tmpEle.appendChild(newEle);
				tmpEle = newEle;
			} else
				tmpEle = (Element) v.get(0);
		}
		return tmpEle;
	}
	
	public static String getTextByPath(Document doc, Element parent,
			String path, String namespaceURI, String attrName) {
		Element ele = openElement(doc, parent, path, namespaceURI);
		if (attrName != null) {
			return ele.getAttributeNS(namespaceURI, attrName);
		} else
			return XMLUtils.getTextValueOfElement(ele);
	}

	public static void setTextByPath(Document doc, Element parent, String path,
			String namespaceURI, String attrName, String value) {
		Element ele = openElement(doc, parent, path, namespaceURI);
		if (attrName != null) {
			ele.setAttributeNS(namespaceURI, attrName, value);
		} else {
						NodeList nl;
			nl = ele.getChildNodes();
			for (int i = 0; i < nl.getLength(); i++)
				if (nl.item(i).getNodeType() == Node.TEXT_NODE)
					ele.removeChild(nl.item(i));
						Text t = doc.createTextNode(value);
			ele.appendChild(t);
		}
	}
	
	public static void saveDocumentToFile(Document doc, String fileName)
			throws Exception {
		Source source = new DOMSource(doc);
		FileOutputStream fileOut = new FileOutputStream(fileName);
		Result result = new StreamResult(fileOut);
		Transformer xformer = null;
		xformer = TransformerFactory.newInstance().newTransformer();
		xformer.setOutputProperty(javax.xml.transform.OutputKeys.ENCODING,
				"GB2312");
		xformer.setOutputProperty(javax.xml.transform.OutputKeys.INDENT, "yes");
		xformer.transform(source, result);
		fileOut.close();
	}
	
	public static void saveDocumentToFileEx(Document doc, String fileName,
			OutputFormat format) throws Exception {
		FileOutputStream fileOut = new FileOutputStream(fileName);
		XMLSerializer xmlOut = new XMLSerializer(fileOut, format);
		xmlOut.serialize(doc);
		fileOut.close();
	}
}
